* This file estimates the lower and upper bounds (1) and  implements a log-linearized 
* Euler equation estimator (2a) and a non-linear GMM-LN estimator (2b.1-2b.4) of the effect 
* of children on the marginal utility of consumption using the PSID, reported in
* "Euler Equation Estimation: Children and Credit Constraints," by 
* Thomas H. Jrgensen, tjo@econ.ku.dk
* December, 2015. STATA SE 13, execution time around one hour (the last part is time consuming)
clear all
set more off
cd C:\Dropbox\Projects\RandomParametersEGM\EEECCC\QE\Accepted\code /*set your directory*/

use data3,clear /*Use the "raw" final dataset from Blundell et al. (2008). Data downloaded from "https://www.aeaweb.org/articles.php?doi=10.1257/aer.98.5.1887"*/
drop if agew>59 | agew<20
drop if seo==1 /* Exclude the SEO/low income sample */

sort person year
xtset person year

* Generate additional variables and restrict sample
gen ndur	  = exp(lc)
replace ndur = . if inlist(year,1978,1979)  /*Looks like something is wrong with the imputed consumption in these years*/
gen income_at = y-ftax 
gen totfood	  = fout+food
gen highskilled = educ<=2 
/* 1 is 0-11 grades (includes those with no schooling, i.e. can't read/write people); 
    2 is 12 grades, i.e. high school w or w/o nonacademic training;
    3 is college dropout, college degree, or collage & advanced/professional degree;
    a missing point denotes na/dk. */
g Dkids = D.kids
g cohort = year - age
bysort cohort age:  egen kids_coh = mean(kids)
sort person year

**************************************
* Find number of households and total observations
drop if missing(log(food))
drop if inlist(year,1987,1988)
xtsum food ndur

********************************************************************************
*** 1. BOUNDS (log-linear Euler equation)
********************************************************************************
foreach var in ndur food {
	preserve
	g Lcon 		= log(`var')
	g DLcon 	= D.Lcon
	g Dkids_coh = D.kids_coh
	drop if missing(DLcon)

	* Split sample
	forvalues educ = 0/1 {
		eststo LogLin_lower_`var'`educ': qui reg DLcon Dkids i.year if agew >= 45 & highskilled==`educ' , robust
		eststo LogLin_upper_`var'`educ': qui ivregress 2sls DLcon i.year (Dkids=Dkids_coh) if agew <= 45 & highskilled==`educ', robust
	}
	restore
}

* Table of PSID results for split sample
estout LogLin_lower_food0 LogLin_upper_food0 LogLin_lower_food1 LogLin_upper_food1, stats(N r2, fmt(0 3) labels("Obs" "R2")) drop(*year*) 
estout LogLin_lower_ndur0 LogLin_upper_ndur0 LogLin_lower_ndur1 LogLin_upper_ndur1, stats(N r2, fmt(0 3) labels("Obs" "R2")) drop(*year*)

********************************************************************************
*** 2. "COMMON" ESTIMATORS
********************************************************************************
gen Linc = log(income_at)
g Lfood = log(food)
g Lndur = log(ndur)
g NumKids   = kids
g NumAdults = ncomp - NumKids /*number in family minus number of children*/
** Real gross interest rate. Based on annual (average) 3 month T-bill and annual (average) infliation calculated from the consumer price index. 
cap drop R
g R = .
replace R = 0.9957 if year == 1978
replace R = 0.9885 if year == 1979
replace R = 0.9785 if year == 1980
replace R = 1.0368 if year == 1981
replace R = 1.0445 if year == 1982
replace R = 1.0539 if year == 1983
replace R = 1.0522 if year == 1984
replace R = 1.0393 if year == 1985
replace R = 1.0407 if year == 1986
replace R = 1.0212 if year == 1987
replace R = 1.0259 if year == 1988
replace R = 1.0328 if year == 1989
replace R = 1.0210 if year == 1990
replace R = 1.0113 if year == 1991
replace R = 1.0040 if year == 1992
g logR = log(R)


***************************************************
* 2a: Attanasio et al. (1999)-type log-liner Euler equation estimation
g BirthCohort = cohort
g agewsq = agew*agew
forvalues educ = 0/1{
	preserve
	keep if highskilled==`educ'

	collapse Linc Lfood Lndur NumKids NumAdults agew agewsq logR, by(BirthCohort year)
	xtset BirthCohort year
	xtsum Lndur Lfood
	g DLfood = D.Lfood
	g DLndur = D.Lndur
	g Dkids  = D.NumKids
	g Dadults= D.NumAdults
	g DLinc = D.Linc

	set more off
	foreach var in DLndur DLfood{
		eststo Att_`var'`educ': qui ivreg2 `var'  (Dkids Dadults =  agew agewsq  L2.(Dkids Dadults `var' DLinc ) L3.(Dkids Dadults `var' DLinc ) L4.(`var' DLinc)), robust
		eststo Att_`var'`educ'R: qui ivreg2 `var'  logR (Dkids Dadults =  agew agewsq  L2.(Dkids Dadults `var' DLinc logR) L3.(Dkids Dadults `var' DLinc logR) L4.(`var' DLinc logR)), robust
	}
	restore
}
estout Att_DLfood0 Att_DLfood0R  Att_DLndur0 Att_DLndur0R Att_DLfood1 Att_DLfood1R Att_DLndur1 Att_DLndur1R, cells("b(fmt(3))" "se(par fmt(3))") ///
	 stats(N r2, fmt(0 3) labels("\hline Obs" " $ R^2 $"))

*****************************************************
* 2b: Alan, Attanasio and Browning (2009): Nonlinear GMM 
cap drop D2kids D1kids R1 R2
g D2kids = F2.kids - kids
g D1kids = F1.kids - kids
g R1 = R
g R2 = F.R1
	
********************************* IMPOSE THE PARAMETERS FROM AAB (2009)	
global BETA .99
global RHO  1.45
global NU   .085
* initial values-loop
global NumInit  30 	/*number of starting points considered*/
global Min 		.01 /*lowest point in scale (delta)*/
global Max 		1 	/*highest point in scale (delta)*/
set maxiter 1000, permanently
*************************
* 2b.1. Only estimate theta
foreach var in food  ndur{
	mat define Q`var' = J(${NumInit},1,.)
	set more off

	cap drop C C1 C2 C1ratio C2ratio
	g C = `var'
	replace C = . if C<=0
	g C1 = F.C
	g C2 = F2.C
	g C1ratio = C1/C
	g C2ratio = C2/C

	preserve
	* Remove "outliers" to stabilize the estimator. 
	drop if C1ratio<.1
	drop if C2ratio<.1
	
	* Loop over initial values:
	local min = 10000 /*initialize the minimum*/
	forvalues init =1/$NumInit{
		disp "2b.1 (`var'): `init' of $NumInit"
		* Set the initial starting values. Scaled is equally spaced between min and max
		scalar scale = ${Min}+( ${Max}-${Min} )/(${NumInit}-1)*(`init'-1)
		mat initial = (1)*scale
		capture{
			eststo GMMln`var'`init': gmm (eq1: ${BETA}*R1*(C1ratio^(-${RHO}))*exp({theta}*D1kids)-exp((${RHO}^2)*${NU})) ///
						(eq2: (${BETA}^2)*R1*R2*exp({theta}*D2kids)*(C2ratio^(-${RHO}))-exp((${RHO}^2)*${NU})) , ///
						 winitial(identity) vce(robust) technique(bfgs)  ///
						from(initial) ///
						deriv(1/theta = D1kids*${BETA}*R1*exp({theta}*D1kids)*(C1ratio^(-${RHO}))) ///  
						deriv(2/theta = D2kids*(${BETA}^2)*R1*R2*exp({theta}*D2kids)*(C2ratio^(-${RHO})) )
			scalar converged = e(converged)
			scalar rank 	 = e(rank)
			mat Q`var'[`init',1] = e(Q)
		}

		* if no error occured check if new miminum, the algorithm converged and rank of covariance is 4 (full). The latter prevents using estimates from flat regions
		if _rc==0 & converged==1 & rank>=1 {
			* Update the mimimum and store the estimates
			if (Q`var'[`init',1]<`min') {
				local min = Q`var'[`init',1]
				mat initial`var' = initial
				eststo GMMln`var'Opt_Theta
			}
		}
	}	
	restore
} 

	estout GMMlnfoodOpt_Theta GMMlnndurOpt_Theta , cells("b(fmt(3))" "se(par fmt(3))") ///
	stats(N Q, fmt(0  %4.0g) labels("\hline Obs." " Obj.")) mlabels(none) collabels(none) ///
	label eqlabel(none) varlabels(beta:_cons "$\beta$" rho:_cons "$\rho$" theta:_cons "$\theta$" nu:_cons "$\nu$")
	
	
	
*************************
* 2b.2. Only estimate theta and nu
foreach var in food  ndur{
	*cap estimates drop GMMln`var'Opt
	mat define Q`var' = J(${NumInit},1,.)
	set more off

	cap drop C C1 C2 C1ratio C2ratio
	g C = `var'
	replace C = . if C<=0
	g C1 = F.C
	g C2 = F2.C
	g C1ratio = C1/C
	g C2ratio = C2/C

	preserve
	* Remove "outliers" to stabilize the estimator. 
	drop if C1ratio<.1
	drop if C2ratio<.1
	
	* Loop over initial values:
	local min = 10000 /*initialize the minimum*/
	forvalues init =1/$NumInit{	
		disp "2b.2 (`var'): `init' of $NumInit"
		* Set the initial starting values. Scaled is equally spaced between min and max
		scalar scale = ${Min}+( ${Max}-${Min} )/(${NumInit}-1)*(`init'-1)
		mat initial = (1,.08)*scale
		capture{
			eststo GMMln`var'`init': gmm (eq1: ${BETA}*R1*(C1ratio^(-${RHO}))*exp({theta}*D1kids)-exp((${RHO}^2)*{nu})) ///
						(eq2: (${BETA}^2)*R1*R2*exp({theta}*D2kids)*(C2ratio^(-${RHO}))-exp((${RHO}^2)*{nu})) , ///
						 winitial(identity) vce(robust) technique(bfgs)  ///
						from(initial) ///
						deriv(1/theta = D1kids*${BETA}*R1*exp({theta}*D1kids)*(C1ratio^(-${RHO}))) ///  
						deriv(2/theta = D2kids*(${BETA}^2)*R1*R2*exp({theta}*D2kids)*(C2ratio^(-${RHO})) ) ///
						deriv(1/nu    = -exp((${RHO}^2)*{nu})*(${RHO}^2) ) ///
						deriv(2/nu    = -exp((${RHO}^2)*{nu})*(${RHO}^2) ) 
			scalar converged = e(converged)
			scalar rank 	 = e(rank)
			mat Q`var'[`init',1] = e(Q)
		}
		
		* if no error occured check if new miminum, the algorithm converged and rank of covariance is 4 (full). The latter prevents using estimates from flat regions
		if _rc==0 & converged==1 & rank==2 {
			* Update the mimimum and store the estimates
			if (Q`var'[`init',1]<`min') {
				local min = Q`var'[`init',1]
				mat initial`var' = initial
				eststo GMMln`var'Opt_ThetaNu
			}
		}
	}	
	restore
} 

	estout GMMlnfoodOpt_Theta GMMlnfoodOpt_ThetaNu GMMlnndurOpt_Theta GMMlnndurOpt_ThetaNu , cells("b(fmt(3))" "se(par fmt(3))") ///
	stats(N Q, fmt(0  %4.0g) labels("\hline Obs." " Obj.")) mlabels(none) collabels(none) ///
	label eqlabel(none) varlabels(beta:_cons "$\beta$" rho:_cons "$\rho$" theta:_cons "$\theta$" nu:_cons "$\nu$")
	
	
*************************
* 2b.3. Only estimate theta, nu and rho
* (note: this might take some time...)
global INST L.R1
foreach var in food  ndur{
	mat define Q`var' = J(${NumInit},1,.)
	set more off

	cap drop C C1 C2 C1ratio C2ratio
	g C = `var'
	replace C = . if C<=0
	g C1 = F.C
	g C2 = F2.C
	g C1ratio = C1/C
	g C2ratio = C2/C

	preserve
	* Remove "outliers" to stabilize the estimator. 
	drop if C1ratio<.1
	drop if C2ratio<.1
	
	* Loop over initial values:
	local min = 10000 /*initialize the minimum*/
	forvalues init =1/$NumInit{	
		disp "2b.3 (`var'): `init' of $NumInit"
		* Set the initial starting values. Scaled is equally spaced between min and max
		scalar scale = ${Min}+( ${Max}-${Min} )/(${NumInit}-1)*(`init'-1)
		mat initial = (2,1,0.08)*scale
		capture{
			eststo GMMln`var'`init': gmm (eq1: ${BETA}*R1*(C1ratio^(-{rho}))*exp({theta}*D1kids)-exp(({rho}^2)*{nu})) ///
						(eq2: (${BETA}^2)*R1*R2*exp({theta}*D2kids)*(C2ratio^(-{rho}))-exp(({rho}^2)*{nu})) , ///
						 instruments(${INST}) winitial(identity) vce(robust) technique(bfgs)  ///
						from(initial) ///
						deriv(1/theta = D1kids*${BETA}*R1*exp({theta}*D1kids)*(C1ratio^(-{rho}))) ///  
						deriv(2/theta = D2kids*(${BETA}^2)*R1*R2*exp({theta}*D2kids)*(C2ratio^(-{rho})) ) ///
						deriv(1/nu    = -exp(({rho}^2)*{nu})*({rho}^2) ) ///
						deriv(2/nu    = -exp(({rho}^2)*{nu})*({rho}^2) ) ///
						deriv(1/rho    = -(C1ratio^(-{rho}))*log(C1ratio)*${BETA}*R1*exp({theta}*D1kids)) ///
						deriv(2/rho    = -(C2ratio^(-{rho}))*log(C2ratio)*(${BETA}^2)*R1*R1*exp({theta}*D2kids))
			scalar converged = e(converged)
			scalar rank 	 = e(rank)
			mat Q`var'[`init',1] = e(Q)
		}
		
		* if no error occured check if new miminum, the algorithm converged and rank of covariance is 4 (full). The latter prevents using estimates from flat regions
		if _rc==0 & converged==1 & rank==3 {
			* Update the mimimum and store the estimates
			if (Q`var'[`init',1]<`min') {
				local min = Q`var'[`init',1]
				mat initial`var' = initial
				eststo GMMln`var'Opt_ThetaNuRho
			}
		}
	}	
	restore
} 

	estout GMMlnfoodOpt_Theta GMMlnfoodOpt_ThetaNu GMMlnfoodOpt_ThetaNuRho GMMlnndurOpt_Theta GMMlnndurOpt_ThetaNu GMMlnndurOpt_ThetaNuRho , cells("b(fmt(3))" "se(par fmt(3))") ///
	stats(N Q, fmt(0  %4.0g) labels("\hline Obs." " Obj.")) mlabels(none) collabels(none) ///
	label eqlabel(none) varlabels(beta:_cons "$\beta$" rho:_cons "$\rho$" theta:_cons "$\theta$" nu:_cons "$\nu$")
	

	
*************************
* 2b.4. Estimate all: theta, nu, rho and beta
* (note: this might take some time...)
global NumInit  60 
foreach var in food  ndur{
	cap estimates drop GMMln`var'Opt
	mat define Q`var' = J(${NumInit},1,.)
	set more off

	cap drop C C1 C2 C1ratio C2ratio
	g C = `var'
	replace C = . if C<=0
	g C1 = F.C
	g C2 = F2.C
	g C1ratio = C1/C
	g C2ratio = C2/C

	preserve
	* Remove "outliers" to stabilize the estimator. 
	drop if C1ratio<.1
	drop if C2ratio<.1
	
	* Loop over initial values:
	local min = 10000 /*initialize the minimum*/
	forvalues init =1/$NumInit{	
		disp "2b.4 (`var'): `init' of $NumInit"
		* Set the initial starting values. Scaled is equally spaced between min and max
		scalar scale = ${Min}+( ${Max}-${Min} )/(${NumInit}-1)*(`init'-1)
		mat initial = (1,2,1,.1)*scale
	
		capture{
			eststo GMMln`var'`init':qui gmm (eq1: {beta}*R1*(C1ratio^(-{rho}))*exp({theta}*D1kids)-exp(({rho}^2)*{nu})) ///
						(eq2: ({beta}^2)*R1*R2*exp({theta}*D2kids)*(C2ratio^(-{rho}))-exp(({rho}^2)*{nu})) , ///
						instruments(L.R1)  winitial(identity) vce(robust) technique(bfgs)  ///
						from(initial) ///
						deriv(1/rho    = -(C1ratio^(-{rho}))*log(C1ratio)*{beta}*R1*exp({theta}*D1kids)) ///
						deriv(1/beta  = R1*exp({theta}*D1kids)*(C1ratio^(-{rho}))) ///
						deriv(1/theta = D1kids*{beta}*R1*exp({theta}*D1kids)*(C1ratio^(-{rho}))) ///
						deriv(1/nu    = -exp(({rho}^2)*{nu})*({rho}^2) ) ///
						deriv(2/rho    = -(C2ratio^(-{rho}))*log(C2ratio)*({beta}^2)*R1*R1*exp({theta}*D2kids)) ///
						deriv(2/beta  = 2*{beta}*R1*R2*exp({theta}*D2kids)*(C2ratio^(-{rho})) ) ///
						deriv(2/theta = D2kids*({beta}^2)*R1*R2*exp({theta}*D2kids)*(C2ratio^(-{rho})) ) ///
						deriv(2/nu    = -exp(({rho}^2)*{nu})*({rho}^2) ) 
			scalar converged = e(converged)
			scalar rank 	 = e(rank)
			mat Q`var'[`init',1] = e(Q)
		}

		* if no error occured check if new miminum, the algorithm converged and rank of covariance is 4 (full). The latter prevents using estimates from flat regions
		if _rc==0 & converged==1 & rank==4 {
			* Update the mimimum and store the estimates
			if (Q`var'[`init',1]<`min') {
				local min = Q`var'[`init',1]
				mat initial`var' = initial
				eststo GMMln`var'Opt
			}
		}
	}	
	restore
} 

* Table combines all the estimation results from the LN-GMM estimations above
	estout GMMlnfoodOpt_Theta GMMlnfoodOpt_ThetaNu GMMlnfoodOpt_ThetaNuRho GMMlnfoodOpt GMMlnndurOpt_Theta GMMlnndurOpt_ThetaNu GMMlnndurOpt_ThetaNuRho GMMlnndurOpt, cells("b(fmt(3))" "se(par fmt(3))") ///
	stats(N Q, fmt(0  %4.0g) labels("\hline Obs." " Obj.")) mlabels(none) collabels(none) ///
	label eqlabel(none) varlabels(beta:_cons "$\beta$" rho:_cons "$\rho$" theta:_cons "$\theta$" nu:_cons "$\nu$")
